/******************************************************************************/
/* Sistemas Integrados TECNISA                                                */
/*                                                                            */
/* Compras e Suprimentos                                                      */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/* Nome: 8240_SUPSTP_PRECO_ECON_AF                                            */
/*                                                                            */
/* Descrio:                                                                 */
/* Calcula o preo econmico para um determinado produto da AF                */
/*                                                                            */
/*                                                                            */
/* Criao:                                                                   */
/* 09/07/99 - Yoris L. de Souza                                               */
/*                                                                            */
/* Manutenes:                                                               */
/* 29/07/1999 - Alteracao na forma de calculo do Preco Economico para Forma de*/
/* pagamento Variavel e arredondamento do resultado dos calculos de Preco Eco.*/
/*                                                                            */
/*                                                                            */
/******************************************************************************/

/******************************************************************************/
/* PARAMETROS:                                                                */
/* P_CDFILIAL    - Codigo da Central de Compras.                              */
/* P_CDFILDEST   - Codigo da Filial Destino                                   */
/* P_CDFILLANC   - Codigo da Filial Lancamento                                */
/* P_NRAUTOFORN  - Numero da AF                                               */
/* P_CDPRODUTO   - Codigo do Produto                                          */
/* P_CDMARCA     - Codigo da Marca                                            */
/* P_DTPROGENTAF - Data de entrega do produto                                 */
/******************************************************************************/

IF EXISTS(SELECT * FROM sysobjects O, sysusers U
          WHERE
            o.name = 'PRECO_ECON_AF'  and
            type = 'P'                  and
            u.uid = o.uid               and 
            u.name = USER)
  DROP PROCEDURE PRECO_ECON_AF
  GO

CREATE PROCEDURE PRECO_ECON_AF @P_CDFILIAL      VARCHAR(4),
                               @P_CDFILDEST     VARCHAR(4),
                               @P_CDFILLANC     VARCHAR(4),
                               @P_NRAUTOFORN    VARCHAR(10),
                               @P_CDPRODUTO     VARCHAR(10),
                               @P_CDMARCA       VARCHAR(5),
                               @P_DTPROGENTAF   DATETIME,
                               @P_VRPRECECO     NUMERIC(15,6) OUTPUT

AS

   DECLARE @V_DTPARCELA    DATETIME;
   DECLARE @V_NRDIAS       NUMERIC(15,6);
   DECLARE @V_NRFATOR      NUMERIC(15,6);
   DECLARE @V_VRINDICE     NUMERIC(15,6);
   DECLARE @V_DTULDIAMES   DATETIME;
   DECLARE @V_DTULTAUX     DATETIME;
   DECLARE @V_FATORPARC    NUMERIC(15,6);
   DECLARE @V_VRPRECECO    NUMERIC(15,6);
   DECLARE @V_VRPZOPGT     NUMERIC(15,6);
   DECLARE @V_CONTA        NUMERIC(15,6);
   DECLARE @V_DTPRIPARC    DATETIME;
   DECLARE @V_TOTREST      NUMERIC(15,6);

   DECLARE @V_DTAUTOFORN   DATETIME;
   DECLARE @V_CDFORMPGTO   VARCHAR(2);
   DECLARE @V_IDTPFORPGTAF VARCHAR(1);
   DECLARE @V_NRPARCPGTOAF SMALLINT;
   DECLARE @V_NRDIAPGFIXAF SMALLINT;
   DECLARE @V_IDUTDIAPGTAF VARCHAR(1);
   DECLARE @V_VRPREUNIAF   NUMERIC(15,5);
   DECLARE @V_VRPERDESAF   NUMERIC(6,2);

   DECLARE @V_VRIMPITM     NUMERIC(16,2);
   DECLARE @V_VRRENDATUMAR NUMERIC(7,3);

BEGIN

   SELECT
    @V_CDFORMPGTO   = AUTOFORNE.CDFORMPGTO,
    @V_IDTPFORPGTAF = AUTOFORNE.IDTPFORPGTAF,
    @V_NRPARCPGTOAF = AUTOFORNE.NRPARCPGTOAF,
    @V_NRDIAPGFIXAF = AUTOFORNE.NRDIAPGFIXAF,
    @V_IDUTDIAPGTAF = AUTOFORNE.IDUTDIAPGTAF,
    @V_DTAUTOFORN   = AUTOFORNE.DTAUTOFORN,
    @V_VRPREUNIAF   = ITEMAUTF.VRPREUNIAF,
    @V_VRPERDESAF   = ITEMAUTF.VRPERDESAF
   FROM AUTOFORNE, ITEMAUTF
   WHERE
    AUTOFORNE.CDFILDEST  = @P_CDFILDEST AND
    AUTOFORNE.CDFILLANC  = @P_CDFILLANC AND
    AUTOFORNE.NRAUTOFORN = @P_NRAUTOFORN AND
    ITEMAUTF.CDFILDEST   = AUTOFORNE.CDFILDEST AND
    ITEMAUTF.CDFILLANC   = AUTOFORNE.CDFILLANC AND
    ITEMAUTF.NRAUTOFORN  = AUTOFORNE.NRAUTOFORN AND
    ITEMAUTF.CDPRODUTO   = @P_CDPRODUTO AND
    ITEMAUTF.CDMARCA     = @P_CDMARCA AND
    ITEMAUTF.DTPROGENTAF = @P_DTPROGENTAF;

   SELECT
     @V_VRRENDATUMAR = MAX(MARCPROD.VRRENDATUMAR)
   FROM MARCPROD
   WHERE
     MARCPROD.CDPRODUTO  =  @P_CDPRODUTO AND
     MARCPROD.CDMARCA    =  @P_CDMARCA;

   SELECT
     @V_VRIMPITM = SUM(VRADIMPAF)
   FROM
     IMPITMAF
   WHERE
     CDFILDEST   = @P_CDFILDEST   AND
     CDFILLANC   = @P_CDFILLANC   AND
     NRAUTOFORN  = @P_NRAUTOFORN  AND
     CDPRODUTO   = @P_CDPRODUTO   AND
     CDMARCA     = @P_CDMARCA     AND
     DTPROGENTAF = DTPROGENTAF;

   IF @V_VRPERDESAF IS NULL
   BEGIN
      SET @V_VRPERDESAF = 0;
   END;
   IF @V_VRRENDATUMAR IS NULL
   BEGIN
      SET @V_VRRENDATUMAR = 0;
   END;
   IF @V_VRIMPITM IS NULL
   BEGIN
      SET @V_VRIMPITM = 0;
   END;
   SET @V_VRPRECECO = 0;

   IF @V_IDTPFORPGTAF = 'V'
   BEGIN
      SELECT
       @V_DTPRIPARC = MIN(DATEADD(DD, ROUND(((CONVERT(NUMERIC, ISNULL(DIAPGTO.NRDIAFINPGTO,1)) - 1)/2),0),
                                      AUTOFORNE.DTAUTOFORN))
      FROM
       DIAPGTO, AUTOFORNE
      WHERE
       DIAPGTO.CDFORMPGTO                      = @V_CDFORMPGTO AND
       CONVERT(NUMERIC, DIAPGTO.NRDIAINIPGTO ) = 1             AND
       AUTOFORNE.CDFILDEST                     = @P_CDFILDEST  AND
       AUTOFORNE.CDFILLANC                     = @P_CDFILLANC  AND
       AUTOFORNE.NRAUTOFORN                    = @P_NRAUTOFORN;

      IF @V_DTPRIPARC IS NULL 
      BEGIN
        SET @V_DTPRIPARC = @V_DTAUTOFORN;
      END;
      SET @V_TOTREST = 0;

      DECLARE C_PARCELAS CURSOR  FOR
      SELECT
       DATEADD(DD,PRAZOPGTAF.NRDIASPGTOAF, @V_DTPRIPARC) AS DTPARCELA,
       PRAZOPGTAF.VRPERCPGTOAF AS VRPZOPGT
      FROM
       PRAZOPGTAF
      WHERE
       PRAZOPGTAF.CDFILDEST     = @P_CDFILDEST AND
       PRAZOPGTAF.CDFILLANC     = @P_CDFILLANC AND
       PRAZOPGTAF.NRAUTOFORN    = @P_NRAUTOFORN;

      OPEN C_PARCELAS;
      FETCH NEXT FROM C_PARCELAS INTO @V_DTPARCELA, @V_VRPZOPGT;

      WHILE (@@FETCH_STATUS = 0)
      BEGIN

         SET @V_NRFATOR = 1;
         SET @V_DTULDIAMES = NULL;
         SET @V_NRDIAS = NULL;
         SELECT
            @V_DTULDIAMES = (CONVERT(DATETIME, '01/' + CONVERT(VARCHAR, DATEPART(MM, DATEADD(MM,1,@V_DTAUTOFORN))) + '/' + CONVERT(VARCHAR, DATEPART(YYYY, DATEADD(MM,1, @V_DTAUTOFORN)))) - 1),
            @V_NRDIAS     = DATEDIFF(DD, @V_DTAUTOFORN, (CONVERT(DATETIME, '01/' + CONVERT(VARCHAR, DATEPART(MM, DATEADD(MM,1,@V_DTAUTOFORN))) + '/' + CONVERT(VARCHAR, DATEPART(YYYY, DATEADD(MM,1, @V_DTAUTOFORN)))) - 1))
         FROM PARAMGERAL;

         SET @V_DTULTAUX = @V_DTAUTOFORN;
         WHILE @V_DTPARCELA > @V_DTULDIAMES
         BEGIN

           SET @V_VRINDICE = NULL;
           SELECT @V_VRINDICE = MAX(MOVINDICE.VRINDICE)
           FROM
            PARASUPRI, MOVINDICE
           WHERE
            PARASUPRI.CDFILIAL   = @P_CDFILIAL               AND
            MONTH(MOVINDICE.DTREFIND) = MONTH(@V_DTULDIAMES) AND
            YEAR(MOVINDICE.DTREFIND)  = YEAR(@V_DTULDIAMES)  AND
            PARASUPRI.CDTPINDICE = MOVINDICE.CDTPINDICE AND
            PARASUPRI.IDPERVALOR = MOVINDICE.IDPERVALOR AND
            PARASUPRI.CDINDICE   = MOVINDICE.CDINDICE;

           IF @V_VRINDICE IS NULL
           BEGIN
              SET @V_VRINDICE = 0;
           END;

           SET @V_NRFATOR = @V_NRFATOR * POWER((1 + (@V_VRINDICE/100)), (@V_NRDIAS/30));

           SET @V_DTULTAUX = @V_DTULDIAMES;
           SET @V_DTULDIAMES = NULL;
           SET @V_NRDIAS     = NULL;
           SELECT
              @V_DTULDIAMES = CONVERT(DATETIME, '01/' + CONVERT(VARCHAR, DATEPART(MM, DATEADD(MM,1,(@V_DTULDIAMES+1)))) + '/' + CONVERT(VARCHAR, DATEPART(YYYY, DATEADD(MM,1, (@V_DTULDIAMES+1))))) - 1,
              @V_NRDIAS     = DATEDIFF(DD, @V_DTULDIAMES, (CONVERT(DATETIME, '01/' + CONVERT(VARCHAR, DATEPART(MM, DATEADD(MM,1,(@V_DTULDIAMES+1)))) + '/' + CONVERT(VARCHAR, DATEPART(YYYY, DATEADD(MM,1, (@V_DTULDIAMES+1))))) - 1))
           FROM PARAMGERAL;
         END;

         SET @V_NRDIAS = DATEDIFF(DD, @V_DTULTAUX, @V_DTPARCELA);

         SET @V_VRINDICE = NULL;
         SELECT @V_VRINDICE = MAX(MOVINDICE.VRINDICE)
         FROM
          PARASUPRI, MOVINDICE
         WHERE
          PARASUPRI.CDFILIAL        = @P_CDFILIAL          AND
          MONTH(MOVINDICE.DTREFIND) = MONTH(@V_DTPARCELA)  AND
          YEAR(MOVINDICE.DTREFIND)  = YEAR(@V_DTPARCELA)   AND
          PARASUPRI.CDTPINDICE      = MOVINDICE.CDTPINDICE AND
          PARASUPRI.IDPERVALOR      = MOVINDICE.IDPERVALOR AND
          PARASUPRI.CDINDICE        = MOVINDICE.CDINDICE;

         IF @V_VRINDICE IS NULL
         BEGIN
            SET @V_VRINDICE = 0;
         END;

         SET @V_NRFATOR = @V_NRFATOR * POWER((1 + (@V_VRINDICE/100)), (@V_NRDIAS/30));

         SET @V_TOTREST = @V_TOTREST + ((1 / @V_NRFATOR) * (@V_VRPZOPGT/100));

         FETCH NEXT FROM C_PARCELAS INTO @V_DTPARCELA, @V_VRPZOPGT;
      END;
      CLOSE C_PARCELAS;
      DEALLOCATE C_PARCELAS;

      SET @V_VRPRECECO = ROUND((@V_VRPREUNIAF - (@V_VRPREUNIAF * (@V_VRPERDESAF /100)) + @V_VRIMPITM) *
                               (1/(1+(@V_VRRENDATUMAR/100))) * @V_TOTREST,5);


   END
   ELSE IF @V_IDTPFORPGTAF = 'F'
   BEGIN

      DECLARE C_PARCFIXAS CURSOR FOR
      SELECT
       CONVERT(DATETIME, (RIGHT(REPLICATE('0',2)+CONVERT(VARCHAR, NRDIAPGFIXAF),2) + '/' +
                          RIGHT(REPLICATE('0',2)+MONTH(DTAUTOFORN),2) + '/' +
                          RIGHT(REPLICATE('0',4)+YEAR(DTAUTOFORN),4)))
      FROM
        AUTOFORNE
      WHERE
       AUTOFORNE.CDFILDEST      = @P_CDFILDEST AND
       AUTOFORNE.CDFILLANC      = @P_CDFILLANC AND
       AUTOFORNE.NRAUTOFORN     = @P_NRAUTOFORN AND
       CONVERT(DATETIME, (RIGHT(REPLICATE('0',2)+CONVERT(VARCHAR, NRDIAPGFIXAF),2) + '/' +
                          RIGHT(REPLICATE('0',2)+MONTH(DTAUTOFORN),2) + '/' +
                          RIGHT(REPLICATE('0',4)+YEAR(DTAUTOFORN),4))) >
       DTAUTOFORN AND
       IDUTDIAPGTAF = 'I'
      ---------------
      UNION
      ---------------
      SELECT
       NRDIAPGFIXAF +
       (CONVERT(DATETIME, '01/' + CONVERT(VARCHAR, DATEPART(MM, DATEADD(MM,1,DTAUTOFORN))) + '/' + CONVERT(VARCHAR, DATEPART(YYYY, DATEADD(MM,1, DTAUTOFORN)))) - 1)
      FROM
       AUTOFORNE
      WHERE
       AUTOFORNE.CDFILDEST      = @P_CDFILDEST AND
       AUTOFORNE.CDFILLANC      = @P_CDFILLANC AND
       AUTOFORNE.NRAUTOFORN     = @P_NRAUTOFORN AND
       CONVERT(DATETIME, (RIGHT(REPLICATE('0',2)+CONVERT(VARCHAR, NRDIAPGFIXAF),2) + '/' +
                          RIGHT(REPLICATE('0',2)+MONTH(DTAUTOFORN),2) + '/' +
                          RIGHT(REPLICATE('0',4)+YEAR(DTAUTOFORN),4))) <=
       DTAUTOFORN AND
       IDUTDIAPGTAF = 'I'
      ---------------
      UNION
      ---------------
      SELECT
       NRDIAPGFIXAF +
       (CONVERT(DATETIME, '01/' + CONVERT(VARCHAR, DATEPART(MM, DATEADD(MM,1,DTAUTOFORN))) + '/' + CONVERT(VARCHAR, DATEPART(YYYY, DATEADD(MM,1, DTAUTOFORN)))) - 1)
      FROM
       AUTOFORNE
      WHERE
       AUTOFORNE.CDFILDEST      = @P_CDFILDEST AND
       AUTOFORNE.CDFILLANC      = @P_CDFILLANC AND
       AUTOFORNE.NRAUTOFORN     = @P_NRAUTOFORN AND
       IDUTDIAPGTAF = 'E';

      OPEN C_PARCFIXAS;
      FETCH NEXT FROM C_PARCFIXAS INTO @V_DTPARCELA;
      CLOSE C_PARCFIXAS;
      DEALLOCATE C_PARCFIXAS;

      SET @V_FATORPARC = 0;
      SET @V_CONTA     = 1;
      SET @V_NRFATOR   = 1;
      SET @V_DTULTAUX  = @V_DTAUTOFORN;

      SET @V_CONTA = 1;
      WHILE @V_CONTA <= @V_NRPARCPGTOAF
      BEGIN
         SET @V_DTULDIAMES = NULL;
         SET @V_NRDIAS     = NULL;
         SELECT
            @V_DTULDIAMES = CONVERT(DATETIME, '01/' + CONVERT(VARCHAR, DATEPART(MM, DATEADD(MM,1,@V_DTAUTOFORN))) + '/' + CONVERT(VARCHAR, DATEPART(YYYY, DATEADD(MM,1, @V_DTAUTOFORN)))) - 1,
            @V_NRDIAS     = DATEDIFF(DD, @V_DTAUTOFORN, (CONVERT(DATETIME, '01/' + CONVERT(VARCHAR, DATEPART(MM, DATEADD(MM,1,@V_DTAUTOFORN))) + '/' + CONVERT(VARCHAR, DATEPART(YYYY, DATEADD(MM,1, @V_DTAUTOFORN)))) - 1))
         FROM PARAMGERAL;

         SET @V_DTULTAUX = @V_DTAUTOFORN;
         WHILE @V_DTPARCELA > @V_DTULDIAMES
         BEGIN

           SET @V_VRINDICE = NULL;
           SELECT @V_VRINDICE = MAX(MOVINDICE.VRINDICE)
           FROM
            PARASUPRI, MOVINDICE
           WHERE
            PARASUPRI.CDFILIAL        = @P_CDFILIAL          AND
            MONTH(MOVINDICE.DTREFIND) = MONTH(@V_DTULDIAMES) AND
            YEAR(MOVINDICE.DTREFIND)  = YEAR(@V_DTULDIAMES)  AND
            PARASUPRI.CDTPINDICE      = MOVINDICE.CDTPINDICE AND
            PARASUPRI.IDPERVALOR      = MOVINDICE.IDPERVALOR AND
            PARASUPRI.CDINDICE        = MOVINDICE.CDINDICE;

           IF @V_VRINDICE IS NULL
           BEGIN
              SET @V_VRINDICE = 0;
           END;

           SET @V_NRFATOR = @V_NRFATOR * POWER((1 + (@V_VRINDICE/100)),(@V_NRDIAS/30));

           SET @V_DTULTAUX = @V_DTULDIAMES;

           SET @V_DTULDIAMES = NULL;
           SET @V_NRDIAS     = NULL;
           SELECT @V_DTULDIAMES = CONVERT(DATETIME, '01/' + CONVERT(VARCHAR, DATEPART(MM, DATEADD(MM,1,(@V_DTULDIAMES+1)))) + '/' + CONVERT(VARCHAR, DATEPART(YYYY, DATEADD(MM,1, (@V_DTULDIAMES+1))))) - 1,
                  @V_NRDIAS = DATEDIFF(DD, @V_DTULDIAMES, (CONVERT(DATETIME, '01/' + CONVERT(VARCHAR, DATEPART(MM, DATEADD(MM,1,(@V_DTULDIAMES+1)))) + '/' + CONVERT(VARCHAR, DATEPART(YYYY, DATEADD(MM,1, (@V_DTULDIAMES+1))))) - 1))
           FROM PARAMGERAL;

         END;

         SET @V_NRDIAS = DATEDIFF(DD, @V_DTULTAUX, @V_DTPARCELA);

         SET @V_VRINDICE = NULL;
         SELECT @V_VRINDICE = MAX(MOVINDICE.VRINDICE)
         FROM
          PARASUPRI, MOVINDICE
         WHERE
          PARASUPRI.CDFILIAL        = @P_CDFILIAL          AND
          MONTH(MOVINDICE.DTREFIND) = MONTH(@V_DTPARCELA)  AND
          YEAR(MOVINDICE.DTREFIND)  = YEAR(@V_DTPARCELA)   AND
          PARASUPRI.CDTPINDICE      = MOVINDICE.CDTPINDICE AND
          PARASUPRI.IDPERVALOR      = MOVINDICE.IDPERVALOR AND
          PARASUPRI.CDINDICE        = MOVINDICE.CDINDICE;

         IF @V_VRINDICE IS NULL
         BEGIN
            SET @V_VRINDICE = 0;
         END;

         SET @V_NRFATOR = @V_NRFATOR * POWER((1 + (@V_VRINDICE/100)), (@V_NRDIAS/30));

         SET @V_FATORPARC = @V_FATORPARC + (1/@V_NRFATOR);

         SET @V_DTPARCELA = NULL;
         SELECT @V_DTPARCELA = DATEADD(MM, 1, @V_DTPARCELA)
         FROM PARAMGERAL;

         SET @V_CONTA = @V_CONTA + 1;

      END;
      SET @V_VRPRECECO = ROUND(((@V_VRPREUNIAF -
                               (@V_VRPREUNIAF * (@V_VRPERDESAF /100)) +
                                @V_VRIMPITM) / @V_NRPARCPGTOAF) *
                                (1/(1+(@V_VRRENDATUMAR/100))) *
                                @V_FATORPARC,5);

   END
   ELSE
   BEGIN
      SET @V_VRPRECECO = ROUND((@V_VRPREUNIAF -
                               (@V_VRPREUNIAF * (@V_VRPERDESAF /100)) +
                                @V_VRIMPITM) *
                               (1/(1+(@V_VRRENDATUMAR/100))),5);
   END;
   SET @P_VRPRECECO = @V_VRPRECECO;
   --RETURN (@V_VRPRECECO);
END;
GO